﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace WebService
{
    class Response
    {

        public string response = null;
        private string request = null;
        private int state = 100;
        private string path = null, fullpath = null;
        private string folder = null;
        private string file = null;
        private string extension = null;
        private string message = null;
        public string header = null;
        public byte[] content = null;
        public string temp = null;
        private string contentType = "Content-Type: text\r\n";

        public Response(string request)
        {
            this.request = request;
            this.response = "Received";
            this.parse();
            this.constructHeader();
        }

        private void parse()
        {
            using (StringReader reader = new StringReader(this.request))
            {
                string line = string.Empty;
                do
                {
                    line = reader.ReadLine();
                    if (line != null)
                    {
                        if (line.StartsWith("GET") || line.StartsWith("POST"))
                        {
                            this.path = line.Split(' ')[1];
                            this.folder = Path.GetDirectoryName(this.path);
                            this.file = Path.GetFileName(this.path);
                            this.extension = Path.GetExtension(this.path).Trim('.');
                            if (this.file == "" || this.extension == "")
                            {
                                this.folder += "\\"+this.file;
                                this.file = Path.GetFileName(Server.defaultFile);
                                this.extension = Path.GetExtension(Server.defaultFile).Trim('.');
                            }
                            this.fullpath = Path.GetFullPath(Server.ROOT + Path.Combine(this.folder, this.file));
                        }
                    }
                } while (line != null && line.Trim() != "");
            }       
        }

        private void constructHeader()
        {
            this.header = "HTTP/1.1 ";
            this.contentType = "Content-Type: ";
            switch (this.extension)
            {
                case "js":
                    contentType += "application/x-javascript";
                    break;
                case "css":
                    this.contentType += "text/css";
                    break;
                case "html":
                    this.contentType += "text/" + this.extension;
                    break;
                case "jpg":
                case "png":
                case "gif":
                    this.contentType += "image/" + this.extension;
                    break;
                case "mp4":
                    this.contentType += "video/" + this.extension;
                    break;
                case "zip":
                    this.contentType += "application/zip";
                    break;
            }
            try
            {
                if (Server.cache.ContainsKey(this.fullpath)) this.content = Server.cache[this.fullpath];
                else
                {
                    this.content = File.ReadAllBytes(this.fullpath);
                    Server.cache.Add(this.fullpath, this.content);
                    Thread remove = new Thread(() => cacheRemove(this.fullpath));
                    remove.Start();
                }
                this.setCode(200);
            } catch (Exception) { setCode(404); }
            
            this.header += this.message + "\r\n" + "Connection: Close" + "\r\n" + this.contentType + "\r\n\r\n";
        }

        public void cacheRemove(string path)
        {
            Thread.Sleep(1000);
            Server.cache.Remove(path);
        }

        private void setCode(int id)
        {
            switch(id)
            {
                case 404:
                    this.message = "404 Not Found";
                    this.state = 404;
                    break;
                case 200:
                    this.message = "200 OK";
                    this.state = 200;
                    break;
            }
        }
    }
}
